/*****************************************************************************\
 * TP CPE, 3ETI, TP synthese d'images
 * --------------
 *
 * Programme principal des appels OpenGL
\*****************************************************************************/



#include <iostream>
#include <cstdlib>
#include <cstdio>
#include <cmath>

#define GLEW_STATIC 1
#include <GL/glew.h>
#include <GL/glut.h>

#include "glutils.hpp"

#include "mat4.hpp"
#include "vec3.hpp"
#include "vec2.hpp"
#include "triangle_index.hpp"
#include "vertex_opengl.hpp"
#include "image.hpp"
#include "scene.hpp"
#include "info_object_opengl.hpp"
#include "mesh.hpp"
#include "opengl_aide_affichage.hpp"


/*****************************************************************************\
 * Variables globales
 *
 *
\*****************************************************************************/


//identifiant du shader
GLuint shader_program_id;

//variable temporelle
int temps_courant=0;

//les parametres de translations
float translation_x=0.0f;
float translation_y=0.0f;
float translation_z=3.0f;

float angle_x=0.0f;
float angle_y=0.0f;

//Matrice de rotation
mat4 rotation;

//Matrice de rotation animee pour le monstre
mat4 rotation_armadillo;

//Matrice de projection
mat4 projection;


info_object_opengl info_object_1;
info_object_opengl info_object_2;
info_object_opengl info_object_3;
info_object_opengl info_object_4;
info_object_opengl info_object_5;

/*****************************************************************************\
 * Fonctions GLUT
 *
 *
\*****************************************************************************/


/*****************************************************************************\
 * init                                                                      *
\*****************************************************************************/
static void init()
{
    // Chargement du shader
    shader_program_id = read_shader("shader.vert", "shader.frag");

    //matrice de projection
    projection=matrice_projection(50.0f*M_PI/180.0f,1.0f,0.5f,10.0f);
    glUniformMatrix4fv(get_uni_loc(shader_program_id,"projection"),1,false,pointeur(projection)); PRINT_OPENGL_ERROR();

    chargement_objet_1(&info_object_1,shader_program_id);
    chargement_objet_2(&info_object_2,shader_program_id);
    chargement_objet_3(&info_object_3,shader_program_id);
    chargement_objet_4(&info_object_4,shader_program_id);
    chargement_objet_5(&info_object_5,shader_program_id);


    //activation de la gestion de la profondeur
    glEnable(GL_DEPTH_TEST); PRINT_OPENGL_ERROR();

}





//Fonction d'affichage
static void display_callback()
{
    //effacement des couleurs du fond d'ecran
    glClearColor(0.5f, 0.6f, 0.9f, 1.0f); PRINT_OPENGL_ERROR();
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); PRINT_OPENGL_ERROR();


    float u0=(temps_courant%100)/100.0f;
    float u1=(temps_courant%900)/900.0f;
    rotation_armadillo=matrice_rotation(0.05*cos(u0*M_PI*2.0f),0.0f,0.0f,1.0f);
    rotation_armadillo=rotation_armadillo*matrice_rotation(u1*M_PI*2.0f,0.0f,1.0f,0.0f);

    //affichage
    affiche_objet(info_object_1,shader_program_id,rotation,vec3(translation_x,translation_y,translation_z));
    affiche_objet(info_object_2,shader_program_id,mat4(),vec3(0.0f,0.0f,0.0f));
    affiche_objet(info_object_3,shader_program_id,rotation_armadillo,vec3(-0.5f,-0.22f,2.5f));
    affiche_objet(info_object_4,shader_program_id,mat4(),vec3(0.4f,-0.5f,2.0f));
    affiche_objet(info_object_5,shader_program_id,mat4(),vec3(0.4f,-0.5f,2.0f));

    //Changement de buffer d'affichage pour eviter un effet de scintillement
    glutSwapBuffers();
}


/*****************************************************************************\
 * keyboard_callback                                                         *
\*****************************************************************************/
static void keyboard_callback(unsigned char key, int, int)
{
    float d_angle=0.1f;
    float dz=0.1f;

    //quitte le programme si on appuie sur les touches 'q', 'Q', ou 'echap'
    switch (key)
    {
    case 'q':
    case 'Q':
    case 27:
        exit(0);
        break;

    case 'i':
        angle_x+=d_angle;
        break;
    case 'k':
        angle_x-=d_angle;
        break;
    case 'j':
        angle_y+=d_angle;
        break;
    case 'l':
        angle_y-=d_angle;
        break;

    case 'p':
        translation_z+=dz;
        break;
    case 'm':
        translation_z-=dz;
        break;
    }

    mat4 rotation_x=matrice_rotation(angle_x,1.0f,0.0f,0.0f);
    mat4 rotation_y=matrice_rotation(angle_y,0.0f,1.0f,0.0f);
    rotation=rotation_x*rotation_y;
}

/*****************************************************************************\
 * special_callback                                                          *
\*****************************************************************************/
static void special_callback(int key, int,int)
{
    float dL=0.03f;
    switch (key)
    {
    case GLUT_KEY_UP:
        translation_y+=dL; //rotation avec la touche du haut
        break;
    case GLUT_KEY_DOWN:
        translation_y-=dL; //rotation avec la touche du bas
        break;
    case GLUT_KEY_LEFT:
        translation_x-=dL; //rotation avec la touche de gauche
        break;
    case GLUT_KEY_RIGHT:
        translation_x+=dL; //rotation avec la touche de droite
        break;
    }

    //reactualisation de l'affichage
    glutPostRedisplay();
}


/*****************************************************************************\
 * timer_callback                                                            *
\*****************************************************************************/
static void timer_callback(int)
{
    //demande de rappel de cette fonction dans 25ms
    glutTimerFunc(25, timer_callback, 0);

    //avancement du temps
    temps_courant++;

    //reactualisation de l'affichage
    glutPostRedisplay();
}

int main(int argc, char** argv)
{
    //**********************************************//
    //Lancement des fonctions principales de GLUT
    //**********************************************//

    //initialisation
    glutInit(&argc, argv);

    //Mode d'affichage (couleur, gestion de profondeur, ...)
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

    //Taille de la fenetre a l'ouverture
    glutInitWindowSize(600, 600);

    //Titre de la fenetre
    glutCreateWindow("OpenGL");

    //Fonction de la boucle d'affichage
    glutDisplayFunc(display_callback);

    //Fonction de gestion du clavier
    glutKeyboardFunc(keyboard_callback);

    //Fonction des touches speciales du clavier (fleches directionnelles)
    glutSpecialFunc(special_callback);

    //Fonction d'appel d'affichage en chaine
    glutTimerFunc(25, timer_callback, 0);

    //Initialisation des fonctions OpenGL
    glewInit();

    //Notre fonction d'initialisation des donnees et chargement des shaders
    init();


    //Lancement de la boucle (infinie) d'affichage de la fenetre
    glutMainLoop();

    //Plus rien n'est execute apres cela

    return 0;
}

