
#include <GL/glew.h>

#include "scene.hpp"

#include "info_object_opengl.hpp"
#include "vec3.hpp"
#include "vec2.hpp"
#include "mat4.hpp"
#include "vertex_opengl.hpp"
#include "triangle_index.hpp"
#include "image.hpp"
#include "mesh.hpp"
#include "opengl_aide_affichage.hpp"
#include "glutils.hpp"

#include <cassert>
#include <cmath>




void chargement_objet_1(info_object_opengl* info_object,GLuint shader_id)
{
    assert(info_object!=NULL);

    //coordonnees geometriques des sommets
    vec3 p0=vec3(0.0f,0.0f,0.0f);
    vec3 p1=vec3(1.0f,0.0f,0.0f);
    vec3 p2=vec3(0.0f,1.0f,0.0f);
    vec3 p3=vec3(0.8f,0.8f,0.5f);

    //normales pour chaque sommet
    vec3 n0=vec3(0.0f,0.0f,1.0f);
    vec3 n1=vec3(-0.25f,-0.25f,0.8535f);
    vec3 n2=vec3(-0.25f,-0.25f,0.8535f);
    vec3 n3=vec3(-0.5f,-0.5f,0.707);

    //couleur pour chaque sommet
    vec3 c0=vec3(0.0f,0.0f,0.0f);
    vec3 c1=vec3(1.0f,0.0f,0.0f);
    vec3 c2=vec3(0.0f,1.0f,0.0f);
    vec3 c3=vec3(1.0f,1.0f,0.0f);

    //texture du sommet
    vec2 t0=vec2(0.0f,0.0f);
    vec2 t1=vec2(1.0f,0.0f);
    vec2 t2=vec2(0.0f,1.0f);
    vec2 t3=vec2(1.0f,1.0f);

    vertex_opengl v0=vertex_opengl(p0,n0,c0,t0);
    vertex_opengl v1=vertex_opengl(p1,n1,c1,t1);
    vertex_opengl v2=vertex_opengl(p2,n2,c2,t2);
    vertex_opengl v3=vertex_opengl(p3,n3,c3,t3);


    //tableau entrelacant coordonnees-normales
    vertex_opengl geometrie[]={v0,v1,v2,v3};


    //indice des triangles
    triangle_index tri0=triangle_index(0,1,2);
    triangle_index tri1=triangle_index(1,3,2);
    triangle_index index[]={tri0,tri1};

    info_object->nbr_triangle=2;


    //attribution d'un buffer de donnees (1 indique la création d'un buffer)
    glGenBuffers(1,&info_object->vbo); PRINT_OPENGL_ERROR();
    //affectation du buffer courant
    glBindBuffer(GL_ARRAY_BUFFER,info_object->vbo); PRINT_OPENGL_ERROR();
    //copie des donnees des sommets sur la carte graphique
    glBufferData(GL_ARRAY_BUFFER,sizeof(geometrie),geometrie,GL_STATIC_DRAW); PRINT_OPENGL_ERROR();



    //attribution d'un autre buffer d'indice
    glGenBuffers(1,&info_object->vboi); PRINT_OPENGL_ERROR();
    //affectation du buffer courant (buffer d'indice)
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,info_object->vboi); PRINT_OPENGL_ERROR();
    //copie des indices sur la carte graphique
    glBufferData(GL_ELEMENT_ARRAY_BUFFER,sizeof(index),index,GL_STATIC_DRAW); PRINT_OPENGL_ERROR();


    charge_image("brad.tga",info_object,shader_id);

}





void chargement_objet_2(info_object_opengl* info_object,GLuint shader_id)
{
    assert(info_object!=NULL);

    //coordonnees geometriques des sommets
    vec3 p0=vec3(-2.0f,-0.5f,-2.0f);
    vec3 p1=vec3(-2.0f,-0.5f,+6.0f);
    vec3 p2=vec3(+2.0f,-0.5f,+6.0f);
    vec3 p3=vec3(+2.0f,-0.5f,-2.0f);

    //normales pour chaque sommet
    vec3 n0=vec3(0.0f,1.0f,0.0f);
    vec3 n1=vec3(0.0f,1.0f,0.0f);
    vec3 n2=vec3(0.0f,1.0f,0.0f);
    vec3 n3=vec3(0.0f,1.0f,0.0f);

    //couleur pour chaque sommet
    vec3 c0=vec3(1.0f,1.0f,1.0f);
    vec3 c1=vec3(1.0f,1.0f,1.0f);
    vec3 c2=vec3(1.0f,1.0f,1.0f);
    vec3 c3=vec3(1.0f,1.0f,1.0f);

    //texture du sommet
    vec2 t0=vec2(0.0f,0.0f);
    vec2 t1=vec2(3.0f,0.0f);
    vec2 t2=vec2(3.0f,3.0f);
    vec2 t3=vec2(0.0f,3.0f);

    vertex_opengl v0=vertex_opengl(p0,n0,c0,t0);
    vertex_opengl v1=vertex_opengl(p1,n1,c1,t1);
    vertex_opengl v2=vertex_opengl(p2,n2,c2,t2);
    vertex_opengl v3=vertex_opengl(p3,n3,c3,t3);


    //tableau entrelacant coordonnees-normales
    vertex_opengl geometrie[]={v0,v1,v2,v3};


    //indice des triangles
    triangle_index tri0=triangle_index(0,1,2);
    triangle_index tri1=triangle_index(0,2,3);
    triangle_index index[]={tri0,tri1};

    info_object->nbr_triangle=2;

    //attribution d'un buffer de donnees (1 indique la création d'un buffer)
    glGenBuffers(1,&info_object->vbo); PRINT_OPENGL_ERROR();
    //affectation du buffer courant
    glBindBuffer(GL_ARRAY_BUFFER,info_object->vbo); PRINT_OPENGL_ERROR();
    //copie des donnees des sommets sur la carte graphique
    glBufferData(GL_ARRAY_BUFFER,sizeof(geometrie),geometrie,GL_STATIC_DRAW); PRINT_OPENGL_ERROR();



    //attribution d'un autre buffer de donnees
    glGenBuffers(1,&info_object->vboi); PRINT_OPENGL_ERROR();
    //affectation du buffer courant (buffer d'indice)
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,info_object->vboi); PRINT_OPENGL_ERROR();
    //copie des indices sur la carte graphique
    glBufferData(GL_ELEMENT_ARRAY_BUFFER,sizeof(index),index,GL_STATIC_DRAW); PRINT_OPENGL_ERROR();


    charge_image("grass.tga",info_object,shader_id);
}

void chargement_objet_3(info_object_opengl* info_object,GLuint shader_id)
{
    mesh m=load_off_file("armadillo_light.off");


    charge_image("white.tga",info_object,shader_id);


    //transform vertices
    mat4 T=mat4(0.005f,0.00f,0.00f,0.00f,
                0.00f,0.005f,0.00f,0.00f,
                0.00f,0.00f,0.005f,0.00f,
                0.00f,0.00f,0.00f,1.00f);
    mat4 R=matrice_rotation(90.0f*M_PI/180.0f,1.0f,0.0f,0.0f);

    apply_deformation(&m,R*T);
    update_normals(&m);

    fill_color(&m,vec3(0.9f,0.9f,0.7f));


    charge_maillage_vbo(info_object,m);
}

void chargement_objet_4(info_object_opengl* info_object,GLuint shader_id)
{
    mesh m=load_obj_file("dress.obj");
    fill_color(&m,vec3(1.0f,1.0f,1.0f));

    //transform vertices
    mat4 T=mat4(0.5f,0.00f,0.00f,0.00f,
                0.00f,0.5f,0.00f,0.00f,
                0.00f,0.00f,0.5f,0.00f,
                0.00f,0.00f,0.00f,1.00f);
    mat4 R1=matrice_rotation(M_PI/2.0f,1.0f,0.0f,0.0f);
    mat4 R2=matrice_rotation(M_PI,0.0f,1.0f,0.0f);

    apply_deformation(&m,R2*R1*T);
    update_normals(&m);
    invert_normals(&m);

    charge_image("tissu.tga",info_object,shader_id);
    charge_maillage_vbo(info_object,m);

}

void chargement_objet_5(info_object_opengl* info_object,GLuint shader_id)
{
    mesh m=load_obj_file("dancer.obj");
    fill_color(&m,vec3(1.0f,1.0f,1.0f));

    //transform vertices
    mat4 T=mat4(0.5f,0.00f,0.00f,0.00f,
                0.00f,0.5f,0.00f,0.00f,
                0.00f,0.00f,0.5f,0.00f,
                0.00f,0.00f,0.00f,1.00f);
    mat4 R1=matrice_rotation(M_PI/2.0f,1.0f,0.0f,0.0f);
    mat4 R2=matrice_rotation(M_PI,0.0f,1.0f,0.0f);

    apply_deformation(&m,R2*R1*T);
    update_normals(&m);
    invert_normals(&m);

    charge_image("white.tga",info_object,shader_id);
    charge_maillage_vbo(info_object,m);
}

